@@ -7,7 +7,7 @@ our @EXPORT = qw(
   is_sym is_token is_rept is_look
   is_tillnot is_str is_expr);
 
-use Spp::Tools qw(is_perl_array is_perl_str);
+use Spp::Builtin;
 
 sub is_atom {
    my $x = shift;
