@@ -1,32 +1,36 @@
 package Spp::Match;
 
 use Exporter;
-our @ISA       = qw(Exporter);
-our @EXPORT_OK = qw(match);
+our @ISA    = qw(Exporter);
+our @EXPORT = qw(match match_rule);
 
 use 5.012;
 no warnings "experimental";
-use Spp::Tools;
-use Spp::Cursor qw(cursor go get_char pre_char cache recover);
+use Spp::Builtin;
+use Spp::Cursor;
 use Spp::IsChar;
-use Spp::IsAtom qw(is_false is_match is_true is_bool is_atom is_sym is_str);
+use Spp::IsAtom;
 use Spp::ToSpp qw(to_spp);
 
 sub match {
    my ($parser, $text, $mode) = @_;
-   my ($door, $table) = @{ $parser }; 
+   my ($door, $table) = @{$parser};
    my $cursor = cursor($text, $table);
    my $door_rule = $cursor->{ns}{$door};
    if (defined $mode) { $cursor->{debug} = $mode }
    my $match = match_rule($door_rule, $cursor);
-   return $match if is_bool($match);
+   if (is_false($match)) {
+      my $max_report = max_report($cursor);
+      return ['false', $max_report];
+   }
+   return $match if is_true($match);
    my $char = first($door);
    return [$door, $match] if is_upper($char);
    return $match if is_lower($char);
    return ['true'] if $char eq '_';
 }
 
-sub _match_rule {
+sub match_rule {
    my ($rule, $cursor) = @_;
    my ($name, $value)  = @{$rule};
    given ($name) {
@@ -46,7 +50,7 @@ sub _match_rule {
       when ('Till') { match_till($value, $cursor) }
       when ('Rtoken') { match_rtoken($value, $cursor) }
       when ('Ctoken') { match_ctoken($value, $cursor) }
-      when ('Ntoken') {  match_ntoken($value, $cursor) }
+      when ('Ntoken') { match_ntoken($value, $cursor) }
       when ('Any') { match_any($value, $cursor) }
       when ('Expr') { match_expr($value, $cursor) }
       when ('Sym') { match_sym($value, $cursor) }
@@ -54,22 +58,24 @@ sub _match_rule {
    }
 }
 
-sub match_rule {
+sub _match_rule {
    my ($x_rule, $cursor) = @_;
    if ($cursor->{debug} == 1) {
       my $name = $x_rule->[0];
+
       # do not need trace rule
-      if ($name ~~ ['Group','Rept','Look',
-         'Branch','Lbranch','Ctoken']) {
+      if ($name ~~
+         ['Group', 'Rept', 'Look', 'Branch', 'Lbranch', 'Ctoken'])
+      {
          return _match_rule($x_rule, $cursor);
       }
-      my $off  = $cursor->{off};
-      my $str  = $cursor->{str};
-      my $rule = to_spp($x_rule);
-      my $flag = '->';
-      my $char = char_to_see(substr($str, $off, 1));
-      my $indent   = ' ' x $cursor->{depth};
-      my $log = "$off $char $indent $flag $rule";
+      my $off    = $cursor->{off};
+      my $str    = $cursor->{str};
+      my $rule   = to_spp($x_rule);
+      my $flag   = '->';
+      my $char   = char_to_see(substr($str, $off, 1));
+      my $indent = ' ' x $cursor->{depth};
+      my $log    = "$off $char $indent $flag $rule";
       $cursor->{depth}++;
       my $match = _match_rule($x_rule, $cursor);
       $cursor->{depth}--;
@@ -85,14 +91,15 @@ sub match_rule {
 sub char_to_see {
    my $char = shift;
    given ($char) {
-     when ("\n") { return '\n' }
-     when ("\r") { return '\r' }
-     when ("\t") { return '\t' }
-     when (" ")  { return '\s' }
-     when (chr(0)) { return '\0' }
-     default { return " $char" }
+      when ("\n")   { return '\n' }
+      when ("\r")   { return '\r' }
+      when ("\t")   { return '\t' }
+      when (" ")    { return '\s' }
+      when (chr(0)) { return '\0' }
+      default       { return " $char" }
    }
 }
+
 sub match_any {
    my ($any, $cursor) = @_;
    my $char = get_char($cursor);
@@ -144,22 +151,23 @@ sub match_branch {
       my $match = match_rule($rule, $cursor);
       if (is_false($match)) {
          recover($cursor, $cache);
-      } else { return $match }
+      }
+      else { return $match }
    }
    return ['false'];
 }
 
 sub match_lbranch {
    my ($branch, $cursor) = @_;
-   my $cache = cache($cursor);
-   my $max_match  = ['false'];
+   my $cache     = cache($cursor);
+   my $max_match = ['false'];
    my $max_cache = $cache;
    for my $rule (@{$branch}) {
       my $match = match_rule($rule, $cursor);
       if (is_match($match)) {
          if ($cursor->{off} >= $max_cache->[1]) {
             $max_cache = cache($cursor);
-            $max_match  = $match;
+            $max_match = $match;
          }
       }
       recover($cursor, $cache);
@@ -179,17 +187,19 @@ sub match_ntoken {
    my $str    = substr($cursor->{str}, $from, $len);
    my $c_name = '$' . $name;
    $cursor->{ns}->{$c_name} = ['Str', $str];
+
    if ($cursor->{debug} == 2) {
       push @{$cache}, $len;
       return name_match($name, $match, $cache);
-   } else {
+   }
+   else {
       return name_match($name, $match);
    }
 }
 
 sub match_ctoken {
    my ($name, $cursor) = @_;
-   my $rule = $cursor->{ns}->{$name};
+   my $rule  = $cursor->{ns}->{$name};
    my $cache = $cursor->{off};
    my $match = match_rule($rule, $cursor);
    return $match if is_bool($match);
@@ -221,17 +231,21 @@ sub match_not {
 
 sub match_till {
    my ($rule, $cursor) = @_;
+
    # say to_json($rule);
    my @buf = ();
+
    # could not reach chr(0) how to match $
    while ($cursor->{off} < $cursor->{len}) {
       my $char = get_char($cursor);
       my $match = match_rule($rule, $cursor);
+
       # say "<$char>";
       if (is_false($match)) {
          push @buf, $char;
          go($cursor);
-      } else {
+      }
+      else {
          return gather_match(join('', @buf), $match);
       }
    }
@@ -344,7 +358,7 @@ sub match_catom {
       when ("Range") { return match_range($value, $char) }
       when ("Cclass") { return is_match_cclass($value, $char) }
       when ("Char")   { return $value eq $char }
-      default { die("unknown chclass node: <$name>") }
+      default         { die("unknown chclass node: <$name>") }
    }
 }
 
@@ -380,7 +394,7 @@ sub is_match_cclass {
       when ('W') { return !is_words($char) }
       when ('x') { return is_xdigit($char) }
       when ('X') { return !is_xdigit($char) }
-      default { die("unknown cclass $cchar") }
+      default    { die("unknown cclass $cchar") }
    }
 }
 
@@ -399,6 +413,7 @@ sub match_expr {
 sub match_sym {
    my ($name, $cursor) = @_;
    my $value = eval_sym($name, $cursor->{ns});
+
    # say to_json($value);
    return match_atom($value, $cursor);
 }
@@ -449,27 +464,27 @@ sub gather_match {
    }
    if (is_perl_str($gather)) { return $match }
    if (is_atom($gather)) {
-      return [ $gather, $match ] if is_atom($match);
-      return [ $gather, @{$match} ];
+      return [$gather, $match] if is_atom($match);
+      return [$gather, @{$match}];
    }
-   return [ @{$gather}, $match ] if is_atom($match);
-   return [ @{$gather}, @{$match} ];
+   return [@{$gather}, $match] if is_atom($match);
+   return [@{$gather}, @{$match}];
 }
 
 sub eval_atom {
    my ($atom, $ns) = @_;
    if (!is_atom($atom)) {
       say to_json($atom);
-      die("not atom to eval $atom")
+      die("not atom to eval $atom");
    }
    my ($name, $value) = @{$atom};
    given ($name) {
-      when ('Str') { return $atom }
-      when ('Sym') { return eval_sym($value, $ns) }
-      when ('Expr') { return eval_expr($value, $ns) }
+      when ('Str')   { return $atom }
+      when ('Sym')   { return eval_sym($value, $ns) }
+      when ('Expr')  { return eval_expr($value, $ns) }
       when ('Array') { return eval_array($value, $ns) }
       default {
-         # say to_json($atom); 
+         # say to_json($atom);
          die("Not implement eval atom $name");
       }
    }
@@ -482,7 +497,7 @@ sub eval_sym {
 }
 
 sub eval_expr {
-   my ($expr, $ns)   = @_;
+   my ($expr, $ns) = @_;
    my $name = shift @{$expr};
    given ($name) {
       when ("push") { return eval_push($expr, $ns) }
@@ -494,7 +509,7 @@ sub eval_expr {
 
 sub eval_array {
    my ($array, $ns) = @_;
-   if (len($array) == 0) { return [ "Array", $array ] }
+   if (len($array) == 0) { return ["Array", $array] }
    my $atoms = eval_atoms($array, $ns);
    return ["Array", $atoms];
 }
@@ -503,12 +518,12 @@ sub eval_push {
    my ($atoms, $ns) = @_;
    my $sym = $atoms->[0];
    if (!is_sym($sym)) {
-     die("push only accept symbol");
+      die("push only accept symbol");
    }
    my $name = $sym->[1];
    my $eval_atoms = eval_atoms($atoms, $ns);
    my ($array, $element) = @{$eval_atoms};
-   push @{$array->[1]}, $element;
+   push @{ $array->[1] }, $element;
    $ns->{$name} = $array;
    return ['true'];
 }
@@ -522,8 +537,9 @@ sub eval_my {
       my $name = $sym->[1];
       $ns->{$name} = $value;
       return ['true'];
-   } else {
-      die("only assign symbol")
+   }
+   else {
+      die("only assign symbol");
    }
 }
 
@@ -548,5 +564,5 @@ sub eval_atoms {
    }
    return $eval_atoms;
 }
-   
+
 1;
