@@ -2,11 +2,22 @@ package Spp::ToSpp;
 
 use Exporter;
 our @ISA    = qw(Exporter);
-our @EXPORT = qw(to_spp);
+our @EXPORT = qw(ast_to_spp to_spp);
 
 use 5.012;
 no warnings "experimental";
-use Spp::Tools qw(str);
+use Spp::Builtin;
+
+sub ast_to_spp {
+   my $ast  = shift;
+   my @strs = ();
+   for my $spec (@{$ast}) {
+      my ($name, $rule) = @{$spec};
+      my $rule_str = to_spp($rule);
+      push @strs, "$name = $rule_str";
+   }
+   return join(";", @strs);
+}
 
 # to str use for trace rule or repl of spp
 sub to_spp {
@@ -36,7 +47,7 @@ sub to_spp {
       when ('Rtoken')   { return $value }
       when ('Sym')      { return $value }
       when ('Int')      { return $value }
-      default { die "unknown atom to Spp: |$name|" }
+      default           { die "unknown atom to Spp: |$name|" }
    }
 }
 
@@ -95,12 +106,12 @@ sub cclass_to_spp {
 sub char_to_spp {
    my $char = shift;
    given ($char) {
-     when ('"')  { return '\\"' }
-     when ("\n") { return '\n'  }
-     when ("\r") { return '\r'  }
-     when ("\t") { return '\t'  }
-     when ("'")  { return "\\'" }
-     default { return "'$char'" }
+      when ('"')  { return '\\"' }
+      when ("\n") { return '\n' }
+      when ("\r") { return '\r' }
+      when ("\t") { return '\t' }
+      when ("'")  { return "\\'" }
+      default     { return "'$char'" }
    }
 }
 
